#ifndef _IMGDOFEFFECTMAKER_H_
#define _IMGDOFEFFECTMAKER_H_


#include <opencv2/core/core.hpp>
#include <opencv2/imgproc/imgproc.hpp>
#include <opencv2/highgui/highgui.hpp>

#include "define.h"
#include "ImageFieldDepthMaker.h"
#include "exif.h"

#include <map>
#include <queue>

#pragma comment(lib, "opencv_core244d.lib")
#pragma comment(lib, "opencv_calib3d244d.lib")
#pragma comment(lib, "opencv_highgui244d.lib")
#pragma comment(lib, "opencv_imgproc244d.lib")

#define DISTANCE_INF	1e8
#define KERNEL_RADIUS	7
#define LEN_DIAM		50
#define AREA_RATE		0.03

using namespace cv;

typedef CQueue<PixPoint> PointQue;

typedef std::map<double, int> CocMap;

template<typename T>
inline T &cvMatPointAt(cv::Mat &src, int i , int j , int c = 0) {
	T *curRow = src.ptr<T>(i);
	return *(curRow + j * src.channels() + c);
}

class ImgDoFEffectMaker {
public:
	ImgDoFEffectMaker() : m_row(0), m_col(0) {}
	~ImgDoFEffectMaker() {}

	bool loadSrcImg(const std::string &src);					// load the source image
	bool loadDepInfo(const PicDOFInfo &dep);					// load the depth info
	void makeDoFEffect(double diam);							// apply the DoF effect, diameter is the diameter of the len
	bool saveDstImg(const std::string &dst);					// save the result image

	bool loadDepInfo(const std::string &file);
	
private:
	Mat m_srcImg;												// source image
	Mat m_dstImg;												// result image

	CMatrix<double> m_dep;										// depth info

	CMatrix<bool> m_vis;

	EXIFInfo m_exifInfo;										// exif info of the image

	PixPoint m_lu;
	PixPoint m_rd;

	int m_row;													
	int m_col;

	double m_f;													// the focal length of the camera
	double m_u;													// the depth of the clear object
	CMatrix<double> m_coc;										// the data of coc
	
	CMatrix<int> m_dis;
	PointQue m_bfsque;
	PixPoint m_vispix[MAX_QUEUE];

	float &depthAt(const PixPoint &p) const;
	bool isInValidArea(const PixPoint &p) const;				// judge if the pix in the valid area
	void initCameraParams(const std::string &src);				// initialize the params of the camera
	double calcCoC(const PixPoint &p, double diam);				// calculate the CoC of one point
	void initCoC(double diam);
	void addEffect(const PixPoint &p);							// add the effect on one point
	void doErosion();
	void doErosionAt(CMatrix<double> &src, CMatrix<double> &dst, const PixPoint &p);

	void doGauss(CMatrix<double> &src);
};

inline bool ImgDoFEffectMaker::loadSrcImg(const string &src) {
	//m_srcImg = imread(src);
	IplImage *img = cvLoadImage(src.c_str());
	if (!img) return false;
	m_srcImg = img;
	initCameraParams(src);
	return true;
}

inline bool ImgDoFEffectMaker::saveDstImg(const string &dst) {
	IplImage img = IplImage(m_dstImg);
	return cvSaveImage(dst.c_str(), &img);
}

inline bool ImgDoFEffectMaker::isInValidArea(const PixPoint &p) const {
	return p.x >= 0 && p.x < m_col && p.y >= 0 && p.y < m_row;
}

inline double ImgDoFEffectMaker::calcCoC(const PixPoint &p, double diam) {
	double z = m_dep[p.y][p.x];
	if (zero(z)) return DISTANCE_INF;
	return fabs(diam * m_f * (m_u - z) / (z * (m_u - m_f))) / 2.0;
}

#endif